import pyvisa
import traceback
from time import sleep

MAX_FREQ = 67e9  # 67 GHz
MIN_FREQ = 50e6  # 50 MHz


class PowerMeter(object):
    """
    Base class for U8488A power meter
    """

    def __init__(self):
        self.__rm = pyvisa.ResourceManager()
        self.__dev_list = self.__rm.list_resources()
        self.__dev = None
        self._frequency = 1e9  # 1 GHz

    @property
    def frequency(self):
        return self._frequency

    @frequency.setter
    def frequency(self, freq):
        if MIN_FREQ <= freq <= MAX_FREQ:
            self._frequency = freq
            if not self.__dev:
                self.__dev.write("FREQ {}Hz".format(freq))
                sleep(1)
        else:
            raise ValueError(
                f"Frequency is not in range! // {MIN_FREQ} <= Freq  <= {MAX_FREQ}"
            )

    def get_device_list(self):
        self.__dev_list = self.__rm.list_resources()
        return self.__dev_list

    def __dev_init(self, device):
        try:
            self.__dev = self.__rm.open_resource(device)
            print(f"{device} is opened!")
            self.__calibration()
            self.__dev.query("*OPC?")
            sleep(1)
            self.__dev.write("FREQ {}Hz".format(self._frequency))
            sleep(1)
            self.__dev.write("INIT:CONT ON")
            self.__dev.query("FETC?")
        except Exception:
            traceback.print_exc()

    def __calibration(self):
        self.__dev.write("SYST:PRES DEF")
        self.__dev.write("CAL:ZERO:AUTO ONCE")
        sleep(2)
        print("Device is calibrated")

    def open_device(self, device):
        self.__dev_init(device)

    def get_power(self):
        if not self.__dev:
            r = self.get_device_list()

            if len(r) > 0:
                print("Using the first availabe measurement device")
                self.__dev_init(r[0])
            else:
                print(
                    "No measurement device present. Please check your USB connection! Refer to pyVisa manual."
                )
                raise Exception("No measurement device present!")

        return float(self.__dev.query("FETC?"))



from setuptools import find_packages, setup
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="u8488a",
    packages=find_packages(),
    version="0.1.2",
    description="Keysight U8488A Powermeter Driver for USRP Power Calibration",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Anıl Gürses",
    author_email="agurses@ncsu.edu",
    url="https://github.com/anilgurses/U8488A_USRP_Calibration.git",
    license="MIT",
    install_requires=[],
    setup_requires=["pyvisa", "pyvisa-py", "pyusb", "pyserial"],
)



from uhd.usrp.cal.visa import VISADevice
from time import sleep


class U8488ADriverUSRP(VISADevice):
    """
    Keysight U8488A VISA Driver for USRP
    """

    def __init__(self, resource):
        self.power_offset = 0
        self.dev = resource
        self.dev.timeout = 25000
        self.dev.write("SYST:PRES DEF")
        self.dev.write("CAL:ZERO:AUTO ONCE")
        sleep(2)
        self.dev.query("*OPC?")
        print("DEVICE SELF CALIBRATED -> STARTING MEASUREMENTS")

    def init_power_meter(self):
        """
        Initialize this device to measure power.
        """
        self.dev.write("INIT:CONT ON")

    def init_signal_generator(self):
        """
        Initialize this device to generate signals.
        """
        raise NotImplementedError()

    def set_frequency(self, freq):
        """
        Set frequency
        """
        print("Frequency is set to {} Hz".format(freq))
        self.dev.write("FREQ {}Hz".format(freq))
        sleep(5.0)
        self.dev.write("INIT:CONT ON")
        self.dev.query("FETC?")

    def get_power_dbm(self):
        """
        Return the received/measured power in dBm (power meter) or return  the
        output power it's currently set to (signal generator).
        """
        # self.dev.write("INIT:CONT ON")
        sleep(0.01)
        return float(self.dev.query("FETC?")) + self.power_offset
